<?php
/**
* 2007-2019 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2019 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Naizfit extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'Naizfit';
        $this->tab = 'others';
        $this->version = '1.2.0';
        $this->author = 'Naiz Bespoke Technologies S.L.';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Naiz Fit');
        $this->description = $this->l('Naiz Fit Prestashop 1.7 module');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('NAIZFIT_LIVE_MODE', false);

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('displayFooter') &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('actionValidateOrder') &&
            $this->registerHook('actionObjectOrderHistoryAddAfter');
    }

    public function uninstall()
    {
        Configuration::deleteByName('NAIZFIT_LIVE_MODE');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitNaizFitModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitNaizFitModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'NAIZFIT_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-lock"></i>',
                        'desc' => $this->l('Naiz Fit partner key'),
                        'name' => 'NAIZFIT_API_KEY',
                        'label' => $this->l('Key'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-cloud"></i>',
                        'desc' => $this->l('Naiz Fit endpoint'),
                        'name' => 'NAIZFIT_ENDPOINT',
                        'label' => $this->l('Endpoint'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'select',
                        'label' => $this->l('Place into'),
                        'required' => true,
                        'options' => array(
                            'query' => array(
                                array('key' => 'header', 'name' => $this->l('Header')),
                                array('key' => 'footer', 'name' => $this->l('Footer'))
                            ),
                            'id' => 'key',
                            'name' => 'name'
                        ),
                        'name' => 'NAIZFIT_JS_POSITION',
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'NAIZFIT_LIVE_MODE' => Configuration::get('NAIZFIT_LIVE_MODE', true),
            'NAIZFIT_API_KEY' => Configuration::get('NAIZFIT_API_KEY', ''),
            'NAIZFIT_ENDPOINT' => Configuration::get('NAIZFIT_ENDPOINT') ?: 'https://backend.production.naiz.fit/api/partners/hook/',
            'NAIZFIT_JS_POSITION' => Configuration::get('NAIZFIT_JS_POSITION') ?: 'footer'
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }
    private function displayJavascript() {
        if (Configuration::hasKey("NAIZFIT_API_KEY")) {
            $key = Configuration::get("NAIZFIT_API_KEY");
            $this->context->smarty->assign(array(
                'naizKey' => $key
            ));
            return $this->display(__FILE__, 'naizScript.tpl');
        }
    }
    public function hookDisplayFooter()
    {
        if (!Configuration::hasKey("NAIZFIT_JS_POSITION") || Configuration::get("NAIZFIT_JS_POSITION") == "footer") {
            return $this->displayJavascript();
        }
    }
    public function hookDisplayHeader()
    {
        if (Configuration::hasKey("NAIZFIT_JS_POSITION") && Configuration::get("NAIZFIT_JS_POSITION") == "header") {
            return $this->displayJavascript();
        }
    }

    function sendOrderDetails($order) {
        $customer = new CustomerCore($order->id_customer);
        $cart = new CartCore($order->id_cart);
        $cart_products = $cart->getProducts();
        $naiz_consumer_hash = array_key_exists("naiz_consumer_hash",$_COOKIE) ? $_COOKIE["naiz_consumer_hash"] : "";
        $currency = new CurrencyCore($order->id_currency);
        $my_currency_iso_code = $currency->iso_code;
        $shipping_address = new AddressCore($order->id_address_delivery);
        $billing_address = new AddressCore($order->id_address_invoice);
        if ($shipping_address->id_state != null) {
            $shipping_address->state_name = StateCore::getNameById($shipping_address->id_state);
        }
        if ($billing_address->id_state != null) {
            $billing_address->state_name = StateCore::getNameById($billing_address->id_state);
        }
        $context = array (
            "partner_hash" => Configuration::get('NAIZFIT_API_KEY', ''), 
            "consumer_hash" => $naiz_consumer_hash , 
            "domain" => $_SERVER["HTTP_HOST"], 
            "currency_code" => $my_currency_iso_code,
            "shipping_address" => $shipping_address,
            "billing_address" => $billing_address,
            "order" => $order, 
            "customer" => $customer,
            "cart_products" => $cart_products
        );
        $context_str = json_encode($context);
        $ch = curl_init( (Configuration::get('NAIZFIT_ENDPOINT')?:'https://backend.production.naiz.fit/api/partners/hook/').$_SERVER["HTTP_HOST"] );
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $context_str);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT,10);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(                                                                          
            'Content-Type: application/json',                                                                                
            'Content-Length: '.strlen($context_str))                                                                       
        );
        $response = curl_exec( $ch );
    }

    public function hookActionValidateOrder (array $params){
        $order = $params['order'];
        $this->sendOrderDetails($order);
    }

    public function hookActionObjectOrderHistoryAddAfter (array $params){
        $orderHistory = $params['object'];
        $order = new Order($orderHistory->id_order);
        $this->sendOrderDetails($order);
    }
}
